/*
 * lcd-ip.c
 *
 *  Created on: Feb 20, 2013
 *      Author: ed.nash@timesys.com
 *
 *  Displays the ip address on the frambuffer so dhcp can be used without console
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>

#define DISPLAY_TOP 30
#define DISPLAY_LEFT 30

#define SEGMENT_WIDTH 3
#define SEGMENT_VERT_LENGTH 30
#define SEGMENT_HORIZ_LENGTH 16
#define SEGMENT_SPACE 9

#define DOT_SIDE 10
#define DOT_SPACE 5
#define DOT_TOP (SEGMENT_VERT_LENGTH *2 - DOT_SIDE)

static unsigned char draw_color[] = {0xff, 0xff, 0xff};
static struct fb_var_screeninfo vinfo;
static struct fb_fix_screeninfo finfo;
static char * fb_data;

void draw_bar(int top, int left, int width, int height) {

	int x, y;

	for(x=0; x<width; x++) {
		for(y=0; y<height; y++) {

			char *start = fb_data;
			start += ((top + y) * finfo.line_length);
			start += (left + x) * vinfo.bits_per_pixel/8;

			*(start++) = draw_color[0];
			*(start++) = draw_color[1];
			*(start++) = draw_color[2];
		}
	}
}

void draw_digit(int top, int left, int digit) {

	// these are the bit masks for 7 segment display of each digit
	unsigned char digits[] = {0x7e, 0x30, 0x6d, 0x79, 0x33, 0x5b, 0x5f, 0x70, 0x7f, 0x73};

	if(digits[digit] & 0x40) draw_bar(top, left, SEGMENT_HORIZ_LENGTH, SEGMENT_WIDTH);
	if(digits[digit] & 0x20) draw_bar(top, left + SEGMENT_HORIZ_LENGTH, SEGMENT_WIDTH, SEGMENT_VERT_LENGTH);
	if(digits[digit] & 0x10) draw_bar(top + SEGMENT_VERT_LENGTH, left + SEGMENT_HORIZ_LENGTH, SEGMENT_WIDTH, SEGMENT_VERT_LENGTH);
	if(digits[digit] & 0x8) draw_bar(top + 2*SEGMENT_VERT_LENGTH, left, SEGMENT_HORIZ_LENGTH, SEGMENT_WIDTH);
	if(digits[digit] & 0x4) draw_bar(top + SEGMENT_VERT_LENGTH, left, SEGMENT_WIDTH, SEGMENT_VERT_LENGTH);
	if(digits[digit] & 0x2) draw_bar(top, left, SEGMENT_WIDTH, SEGMENT_VERT_LENGTH);
	if(digits[digit] & 0x1) draw_bar(top + SEGMENT_VERT_LENGTH, left, SEGMENT_HORIZ_LENGTH, SEGMENT_WIDTH);
}

static struct ifreq ifr;

char* get_ip() {

	int fd = socket(AF_INET, SOCK_DGRAM, 0);

	/* I want to get an IPv4 IP address */
	ifr.ifr_addr.sa_family = AF_INET;

	/* I want IP address attached to "eth0" */
	strncpy(ifr.ifr_name, "eth0", IFNAMSIZ - 1);

	ioctl(fd, SIOCGIFADDR, &ifr);

	close(fd);

	/* display result */
	return &ifr.ifr_addr.sa_data[2];
}

int main(int argc, char** argv) {

	int i, left, d1, d2;
	char *ip;

	int fb = open("/dev/fb0", O_RDWR);
	if (fb < 0) {
		perror("cant open /dev/fb0");
		return -1;
	}

	if (ioctl(fb, FBIOGET_FSCREENINFO, &finfo)) {
		perror("FBIOGET_FSCREENINFO");
		return -1;
	}

	if (ioctl(fb, FBIOGET_VSCREENINFO, &vinfo)) {
		perror("FBIOGET_VSCREENINFO");
		return -1;
	}

	unsigned int memsize = finfo.line_length * vinfo.yres;
	fb_data = (char *) mmap(0, memsize, PROT_READ | PROT_WRITE, MAP_SHARED, fb, 0);
	if (fb_data == (void *) -1) {
		perror("mmap");
		return -1;
	}

	memset(fb_data, 0, memsize);

	ip = get_ip();
	left = DISPLAY_LEFT;

	for(i=0; i<4; i++) {

		d1 = ip[i]/100;
		if(d1 > 0) {
			draw_digit(DISPLAY_TOP, left, d1);
			left += SEGMENT_SPACE + SEGMENT_HORIZ_LENGTH;
		}

		d2 = (ip[i] - d1 *100) / 10;

		if((d2 > 0) || (d1 > 0)) {
			draw_digit(DISPLAY_TOP, left, d2);
			left += SEGMENT_SPACE + SEGMENT_HORIZ_LENGTH;
		}

		draw_digit(DISPLAY_TOP, left, (ip[i] - (d1 *100) - (d2 * 10)));
		left += SEGMENT_SPACE + SEGMENT_HORIZ_LENGTH;

		if(i < 3) {
			draw_bar(DISPLAY_TOP + DOT_TOP, left+DOT_SPACE, DOT_SIDE, DOT_SIDE);
			left += (DOT_SPACE+DOT_SIDE)*2;
		}
	}


	close(fb);
	munmap(fb_data, memsize);

	return 0;
}
